@@ -37,16 +37,25 @@ __all__ = [
     'CSSMediaRule',
     'CSSNamespaceRule',
     'CSSPageRule',
+    'MarginRule',
     'CSSStyleRule',
     'CSSUnknownRule',
     'CSSVariablesRule',
     'CSSVariablesDeclaration',
     'Selector', 'SelectorList',
     'CSSStyleDeclaration', 'Property',
-    'CSSValue', 'CSSPrimitiveValue', 'CSSValueList'
+    #'CSSValue', 'CSSPrimitiveValue', 'CSSValueList'
+    'PropertyValue',
+           'Value',
+           'ColorValue',
+           'DimensionValue',
+           'URIValue',
+           'CSSFunction', 
+           'CSSVariable',
+           'MSValue'
     ]
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: __init__.py 1928 2010-03-20 22:40:37Z cthedot $'
+__version__ = '$Id$'
 
 from cssstylesheet import *
 from cssrulelist import *
@@ -58,6 +67,7 @@ from cssimportrule import *
 from cssmediarule import *
 from cssnamespacerule import *
 from csspagerule import *
+from marginrule import *
 from cssstylerule import *
 from cssvariablesrule import *
 from cssunknownrule import *
@@ -66,4 +76,5 @@ from selectorlist import *
 from cssstyledeclaration import *
 from cssvariablesdeclaration import *
 from property import *
-from cssvalue import *
+#from cssvalue import *
+from value import *
