@@ -9,6 +9,7 @@ __all__ = ['PropertyValue',
            'DimensionValue',
            'URIValue',
            'CSSFunction',
+           'CSSCalc',
            'CSSVariable',
            'MSValue'
            ]
@@ -139,18 +140,21 @@ class PropertyValue(cssutils.util._NewBase):
                       _DimensionProd(self, nextSor),
                       _URIProd(self, nextSor),
                       _ValueProd(self, nextSor),
-#                      _CalcValueProd(self, nextSor),
 #                      _Rect(self, nextSor),
                       # all other functions
                       _CSSVariableProd(self, nextSor),
                       _MSValueProd(self, nextSor),
+                      _CalcValueProd(self, nextSor),
                       _CSSFunctionProd(self, nextSor)
                       )
         operator = Choice(PreDef.S(toSeq=False),
                           PreDef.char('comma', ',',
-                                      toSeq=lambda t, tokens: ('operator', t[1])),
+                                      toSeq=lambda t, tokens: ('operator', t[1]),
+                                      optional=True
+                                      ),
                           PreDef.char('slash', '/',
-                                      toSeq=lambda t, tokens: ('operator', t[1])),
+                                      toSeq=lambda t, tokens: ('operator', t[1]),
+                                      optional=True),
                           optional=True)
         prods = Sequence(term,
                          Sequence(# mayEnd this Sequence if whitespace
@@ -169,9 +173,15 @@ class PropertyValue(cssutils.util._NewBase):
                                                     prods)
         # must be at least one value!
         ok = ok and len(list(self.__items(seq))) > 0
+
+        for item in seq:
+            if hasattr(item.value, 'wellformed') and not item.value.wellformed:
+                ok = False
+                break
+
+        self.wellformed = ok
         if ok:
             self._setSeq(seq)
-            self.wellformed = True
         else:
             self._log.error(u'PropertyValue: Unknown syntax or no value: %s' %
                             self._valuestr(cssText))
@@ -222,6 +232,7 @@ class Value(cssutils.util._NewBase):
     HASH = u'HASH'
 
     FUNCTION = u'FUNCTION'
+    CALC = u'CALC'
     VARIABLE = u'VARIABLE'
 
     _type = None
@@ -231,6 +242,7 @@ class Value(cssutils.util._NewBase):
         super(Value, self).__init__()
 
         self.parent = parent
+        self.wellformed = False
 
         if cssText:
             self.cssText = cssText
@@ -254,13 +266,13 @@ class Value(cssutils.util._NewBase):
                        PreDef.unicode_range(stop=True),
                        )
         ok, seq, store, unused = ProdParser().parse(cssText, u'Value', prods)
+        self.wellformed = ok
         if ok:
             # only 1 value anyway!
             self._type = seq[0].type
             self._value = seq[0].value
 
             self._setSeq(seq)
-            self.wellformed = ok
 
     cssText = property(lambda self: cssutils.ser.do_css_Value(self),
                        _setCssText,
@@ -326,7 +338,7 @@ class ColorValue(Value):
                                                  v in (u'rgb(', u'hsl('),
                               toSeq=lambda t, tokens: (t[0], normalize(t[1]))),
                           component,
-                          Sequence(PreDef.comma(),
+                          Sequence(PreDef.comma(optional=True),
                                    component,
                                    minmax=lambda: (2, 2)
                                    ),
@@ -339,7 +351,7 @@ class ColorValue(Value):
                                  normalize(t[1]))
                               ),
                           component,
-                          Sequence(PreDef.comma(),
+                          Sequence(PreDef.comma(optional=True),
                                    component,
                                    minmax=lambda: (3, 3)
                                    ),
@@ -359,6 +371,7 @@ class ColorValue(Value):
         ok, seq, store, unused = ProdParser().parse(cssText,
                                                     self.type,
                                                     prods)
+        self.wellformed = ok
         if ok:
             t, v = seq[0].type, seq[0].value
             if u'IDENT' == t:
@@ -439,7 +452,6 @@ class ColorValue(Value):
             self._colorType = t
             self._red, self._green, self._blue, self._alpha = tuple(rgba)
             self._setSeq(seq)
-            self.wellformed = ok
 
     cssText = property(lambda self: cssutils.ser.do_css_ColorValue(self),
                        _setCssText,
@@ -474,11 +486,11 @@ class ColorValue(Value):
 
 class DimensionValue(Value):
     """
-    A numerical value with an optional dimenstion like e.g. "px" or "%".
+    A numerical value with an optional dimension like e.g. "px" or "%".
 
     Covers DIMENSION, PERCENTAGE or NUMBER values.
     """
-    __reNumDim = re.compile(ur'^(\d*\.\d+|\d+)(.*)$', re.I | re.U | re.X)
+    __reUnNumDim = re.compile(ur'^([+-]?)(\d*\.\d+|\d+)(.*)$', re.I | re.U | re.X)
     _dimension = None
     _sign = None
 
@@ -491,7 +503,7 @@ class DimensionValue(Value):
     def _setCssText(self, cssText):
         self._checkReadonly()
 
-        prods = Sequence(PreDef.unary(),
+        prods = Sequence(#PreDef.unary(),
                          Choice(PreDef.dimension(stop=True),
                                 PreDef.number(stop=True),
                                 PreDef.percentage(stop=True)
@@ -500,34 +512,27 @@ class DimensionValue(Value):
         ok, seq, store, unused = ProdParser().parse(cssText,
                                                     u'DimensionValue',
                                                     prods)
+        self.wellformed = ok
         if ok:
-            sign = val = u''
-            dim = type_ = None
+            item = seq[0]
 
-            # find
-            for item in seq:
-                if item.value in u'+-':
-                    sign = item.value
-                else:
-                    type_ = item.type
-
-                    # number + optional dim
-                    v, d = self.__reNumDim.findall(
+            sign, v, d = self.__reUnNumDim.findall(
                                 normalize(item.value))[0]
-                    if u'.' in v:
-                        val = float(sign + v)
-                    else:
-                        val = int(sign + v)
-                    if d:
-                        dim = d
+            if u'.' in v:
+                val = float(sign + v)
+            else:
+                val = int(sign + v)
+
+            dim = None
+            if d:
+                dim = d
 
             self._sign = sign
             self._value = val
             self._dimension = dim
-            self._type = type_
+            self._type = item.type
 
             self._setSeq(seq)
-            self.wellformed = ok
 
     cssText = property(lambda self: cssutils.ser.do_css_Value(self),
                        _setCssText,
@@ -555,13 +560,13 @@ class URIValue(Value):
         prods = Sequence(PreDef.uri(stop=True))
 
         ok, seq, store, unused = ProdParser().parse(cssText, u'URIValue', prods)
+        self.wellformed = ok
         if ok:
             # only 1 value only anyway
             self._type = seq[0].type
             self._value = seq[0].value
 
             self._setSeq(seq)
-            self.wellformed = ok
 
     cssText = property(lambda self: cssutils.ser.do_css_Value(self),
                        _setCssText,
@@ -603,7 +608,7 @@ class CSSFunction(Value):
                           _DimensionProd(self),
                           _URIProd(self),
                           _ValueProd(self),
-                          #_CalcValueProd(self),
+                          _CalcValueProd(self),
                           _CSSVariableProd(self),
                           _CSSFunctionProd(self)
                           )
@@ -612,7 +617,7 @@ class CSSFunction(Value):
                                   toSeq=lambda t, tokens: (t[0],
                                                            normalize(t[1]))),
                              Choice(Sequence(itemProd,
-                                             Sequence(PreDef.comma(),
+                                             Sequence(PreDef.comma(optional=True),
                                                       itemProd,
                                                       minmax=lambda: (0, None)),
                                              PreDef.funcEnd(stop=True)),
@@ -625,9 +630,9 @@ class CSSFunction(Value):
         ok, seq, store, unused = ProdParser().parse(cssText,
                                                     self.type,
                                                     self._productions())
+        self.wellformed = ok
         if ok:
             self._setSeq(seq)
-            self.wellformed = ok
 
     cssText = property(lambda self: cssutils.ser.do_css_CSSFunction(self),
                        _setCssText,
@@ -659,7 +664,6 @@ class MSValue(CSSFunction):
                                          )
                     )
 
-
         funcProds = Sequence(Prod(name='FUNCTION',
                                   match=lambda t, v: t == types.FUNCTION,
                                   toSeq=lambda t, tokens: (t[0], t[1])
@@ -692,6 +696,84 @@ class MSValue(CSSFunction):
                        doc=u"String value of this value.")
 
 
+class CSSCalc(CSSFunction):
+    """The CSSCalc function represents a CSS calc() function.
+
+    No further API is provided. For multiplication and division no check
+    if one operand is a NUMBER is made.
+    """
+    _functionName = 'CSSCalc'
+
+    def __str__(self):
+        return u"<cssutils.css.%s object at 0x%x>" % (
+                self.__class__.__name__, id(self))
+
+    def _setCssText(self, cssText):
+        self._checkReadonly()
+
+        types = self._prods # rename!
+
+        _operator = Choice(Prod(name='Operator */',
+                                match=lambda t, v: v in u'*/',
+                                toSeq=lambda t, tokens: (t[0], t[1])
+                           ),
+                           Sequence(
+                               PreDef.S(),
+                               Choice(
+                                 Sequence(
+                                    Prod(name='Operator */',
+                                        match=lambda t, v: v in u'*/',
+                                        toSeq=lambda t, tokens: (t[0], t[1])
+                                    ),
+                                    PreDef.S(optional=True)
+                                 ),
+                                 Sequence(
+                                    Prod(name='Operator +-',
+                                        match=lambda t, v: v in u'+-',
+                                        toSeq=lambda t, tokens: (t[0], t[1])
+                                    ),
+                                    PreDef.S()
+                                 ),
+                                 PreDef.funcEnd(stop=True, mayEnd=True)
+                               )
+                           )
+                    )
+
+        _operant = lambda: Choice(_DimensionProd(self),
+                                   _CSSVariableProd(self))
+
+        prods = Sequence(Prod(name='CALC',
+                                  match=lambda t, v: t == types.FUNCTION and
+                                        normalize(v) == u'calc('
+                         ),
+                         PreDef.S(optional=True),
+                         _operant(),
+                         Sequence(_operator,
+                                  _operant(),
+                                  minmax=lambda: (0, None)
+                                  ),
+                         PreDef.funcEnd(stop=True)
+                )
+
+        # store: name of variable
+        ok, seq, store, unused = ProdParser().parse(cssText,
+                                                    u'CSSCalc',
+                                                    prods,
+                                                    checkS=True)
+        self.wellformed = ok
+        if ok:
+            self._setSeq(seq)
+
+
+    cssText = property(lambda self: cssutils.ser.do_css_CSSCalc(self),
+                       _setCssText, doc=u"String representation of calc function.")
+
+    type = property(lambda self: Value.CALC,
+                    doc=u"Type is fixed to Value.CALC.")
+
+
+
+
 class CSSVariable(CSSFunction):
     """The CSSVariable represents a CSS variables like ``var(varname)``.
 
@@ -700,6 +782,7 @@ class CSSVariable(CSSFunction):
     """
     _functionName = 'CSSVariable'
     _name = None
+    _fallback = None
 
     def __str__(self):
         return u"<cssutils.css.%s object name=%r value=%r at 0x%x>" % (
@@ -714,17 +797,34 @@ class CSSVariable(CSSFunction):
                                         normalize(v) == u'var('
                              ),
                              PreDef.ident(toStore='ident'),
+                             Sequence(PreDef.comma(),
+                                      Choice(_ColorProd(self, toStore='fallback'),
+                                             _DimensionProd(self, toStore='fallback'),
+                                             _URIProd(self, toStore='fallback'),
+                                             _ValueProd(self, toStore='fallback'),
+                                             _CalcValueProd(self, toStore='fallback'),
+                                             _CSSVariableProd(self, toStore='fallback'),
+                                             _CSSFunctionProd(self, toStore='fallback')
+                                             ),
+                                      minmax=lambda: (0, 1)
+                                      ),
                              PreDef.funcEnd(stop=True))
 
         # store: name of variable
-        store = {'ident': None}
+        store = {'ident': None, 'fallback': None}
         ok, seq, store, unused = ProdParser().parse(cssText,
                                                     u'CSSVariable',
                                                     prods)
+        self.wellformed = ok
+
         if ok:
             self._name = store['ident'].value
+            try:
+                self._fallback = store['fallback'].value
+            except KeyError:
+                self._fallback = None
+
             self._setSeq(seq)
-            self.wellformed = ok
 
     cssText = property(lambda self: cssutils.ser.do_css_CSSVariable(self),
                        _setCssText, doc=u"String representation of variable.")
@@ -735,6 +835,9 @@ class CSSVariable(CSSFunction):
                         u"a value in a "
                         u":class:`cssutils.css.CSSVariablesDeclaration`.")
 
+    fallback = property(lambda self: self._fallback,
+                doc=u"The fallback Value of this variable")
+
     type = property(lambda self: Value.VARIABLE,
                     doc=u"Type is fixed to Value.VARIABLE.")
 
@@ -761,35 +864,37 @@ class CSSVariable(CSSFunction):
                      doc=u'The resolved actual value or None.')
 
 
+
 # helper for productions
-def _ValueProd(parent, nextSor=False):
+def _ValueProd(parent, nextSor=False, toStore=None):
     return Prod(name='Value',
                 match=lambda t, v: t in ('IDENT', 'STRING', 'UNICODE-RANGE'),
                 nextSor = nextSor,
-                toSeq=lambda t, tokens: ('Value', Value(
-                                            pushtoken(t,
-                                                                      tokens),
+                toStore=toStore,
+                toSeq=lambda t, tokens: ('Value', Value(pushtoken(t, tokens),
                                          parent=parent)
                                          )
                 )
 
 
-def _DimensionProd(parent, nextSor=False):
+def _DimensionProd(parent, nextSor=False, toStore=None):
     return Prod(name='Dimension',
                 match=lambda t, v: t in (u'DIMENSION',
                                          u'NUMBER',
-                                         u'PERCENTAGE') or v in u'+-',
+                                         u'PERCENTAGE'),
                 nextSor = nextSor,
-                toSeq=lambda t, tokens: (t[0], DimensionValue(
+                toStore=toStore,
+                toSeq=lambda t, tokens: (u'DIMENSION', DimensionValue(
                                             pushtoken(t,
-                                                                      tokens),
-                                         parent=parent)
+                                                      tokens),
+                                            parent=parent)
                                          )
                 )
 
-def _URIProd(parent, nextSor=False):
+def _URIProd(parent, nextSor=False, toStore=None):
     return Prod(name='URIValue',
                 match=lambda t, v: t == 'URI',
+                toStore=toStore,
                 nextSor = nextSor,
                 toSeq=lambda t, tokens: ('URIValue', URIValue(
                                             pushtoken(t,
@@ -800,7 +905,7 @@ def _URIProd(parent, nextSor=False):
 
 reHexcolor = re.compile(r'^\#(?:[0-9abcdefABCDEF]{3}|[0-9abcdefABCDEF]{6})$')
 
-def _ColorProd(parent, nextSor=False):
+def _ColorProd(parent, nextSor=False, toStore=None):
     return Prod(name='ColorValue',
                 match=lambda t, v:
                                    (t == 'HASH' and
@@ -816,6 +921,7 @@ def _ColorProd(parent, nextSor=False):
                                     normalize(v) in ColorValue.COLORS.keys()
                                     ),
                 nextSor = nextSor,
+                toStore=toStore,
                 toSeq=lambda t, tokens: ('ColorValue', ColorValue(
                                             pushtoken(t,
                                                                       tokens),
@@ -823,8 +929,9 @@ def _ColorProd(parent, nextSor=False):
                                          )
                 )
 
-def _CSSFunctionProd(parent, nextSor=False):
+def _CSSFunctionProd(parent, nextSor=False, toStore=None):
     return PreDef.function(nextSor=nextSor,
+                           toStore=toStore,
                            toSeq=lambda t, tokens: (CSSFunction._functionName,
                                                     CSSFunction(
                                 pushtoken(t, tokens),
@@ -832,8 +939,21 @@ def _CSSFunctionProd(parent, nextSor=False):
                                 )
                            )
 
-def _CSSVariableProd(parent, nextSor=False):
+def _CalcValueProd(parent, nextSor=False, toStore=None):
+    return Prod(name=CSSCalc._functionName,
+                    match=lambda t, v: t == PreDef.types.FUNCTION and
+                        normalize(v) == u'calc(',
+                    toStore=toStore,
+                    toSeq=lambda t, tokens: (CSSCalc._functionName,
+                                                    CSSCalc(
+                                pushtoken(t, tokens),
+                                parent=parent)
+                                                    ),
+                    nextSor=nextSor)
+
+def _CSSVariableProd(parent, nextSor=False, toStore=None):
     return PreDef.variable(nextSor=nextSor,
+                           toStore=toStore,
                            toSeq=lambda t, tokens: (CSSVariable._functionName,
                                                     CSSVariable(
                                 pushtoken(t, tokens),
@@ -869,3 +989,10 @@ def _MSValueProd(parent, nextSor=False):
                                                  )
                                          )
                 )
+
+
+def MediaQueryValueProd(parent):
+    return Choice(_ColorProd(parent),
+                  _DimensionProd(parent),
+                  _ValueProd(parent),
+                  )
\ No newline at end of file
