@@ -57,6 +57,29 @@ class PropertyValueTestCase(basetest.BaseTestCase):
                  u'a() 1, -1, +1, 1%, -1%, 1px, -1px, "a", a, url(a), #ab4',
                     12, u'a() 1, -1, +1, 1%, -1%, 1px, -1px, "a", a, url(a), #ab4'),
 
+                # calc values
+                 u'calc(1)': (None, 1, u'calc(1)'),
+                 u'calc( 1)': (u'calc(1)', 1, u'calc(1)'),
+                 u'calc(1 )': (u'calc(1)', 1, u'calc(1)'),
+                 u'calc(1px)': (None, 1, u'calc(1px)'),
+                 u'calc(1p-x-)': (None, 1, u'calc(1p-x-)'),
+                 u'calc(1%)': (None, 1, u'calc(1%)'),
+                 u'calc(-1)': (None, 1, u'calc(-1)'),
+                 u'calc(+1)': (None, 1, u'calc(+1)'),
+                 u'calc(1  +   1px)': (u'calc(1 + 1px)', 1, u'calc(1 + 1px)'),
+                 u'calc(1 - 1px)': (None, 1, u'calc(1 - 1px)'),
+                 u'calc(1*1px)': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
+                 u'calc(1  /  1px)': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
+                 u'calc( 1*1px)': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
+                 u'calc( 1  /  1px)': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
+                 u'calc(1*1px )': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
+                 u'calc(1  /  1px )': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
+                 u'calc( 1*1px )': (u'calc(1 * 1px)', 1, u'calc(1 * 1px)'),
+                 u'calc( 1  /  1px )': (u'calc(1 / 1px)', 1, u'calc(1 / 1px)'),
+                 u'calc(var(X))': (None, 1, None),
+                 u'calc(2 * var(X))': (None, 1, None),
+                 u'calc(2px + var(X))': (None, 1, None),
+
                  #issue #24
                  u'rgb(0, 10, 255)': (None, 1, u'rgb(0, 10, 255)'),
                  u'hsl(10, 10%, 25%)': (None, 1, u'hsl(10, 10%, 25%)'),
@@ -208,42 +231,75 @@ class PropertyValueTestCase(basetest.BaseTestCase):
                  }
         self.do_equal_r(tests)
 
-        tests = {   u'a+': xml.dom.SyntaxErr,
-                    u'-': xml.dom.SyntaxErr,
-                    u'+': xml.dom.SyntaxErr,
-                    u'-%': xml.dom.SyntaxErr,
-                    u'+a': xml.dom.SyntaxErr,
-                    u'--1px': xml.dom.SyntaxErr,
-                    u'++1px': xml.dom.SyntaxErr,
-                    u'#': xml.dom.SyntaxErr,
-                    u'#00': xml.dom.SyntaxErr,
-                    u'#12x': xml.dom.SyntaxErr,
-                    u'#xyz': xml.dom.SyntaxErr,
-                    u'#0000': xml.dom.SyntaxErr,
-                    u'#00000': xml.dom.SyntaxErr,
-                    u'#0000000': xml.dom.SyntaxErr,
-                    u'-#0': xml.dom.SyntaxErr,
+        tests = [
+                    u'a+',
+                    u'-',
+                    u'+',
+                    u'-%',
+                    u'+a',
+                    u'--1px',
+                    u'++1px',
+                    u'#',
+                    u'#00',
+                    u'#12x',
+                    u'#xyz',
+                    u'#0000',
+                    u'#00000',
+                    u'#0000000',
+                    u'-#0',
                     # operator
-                    u',': xml.dom.SyntaxErr,
-                    u'1,,2': xml.dom.SyntaxErr,
-                    u'1,/**/,2': xml.dom.SyntaxErr,
-                    u'1  ,  /**/  ,  2': xml.dom.SyntaxErr,
-                    u'1,': xml.dom.SyntaxErr,
-                    u'1, ': xml.dom.SyntaxErr,
-                    u'1 ,': xml.dom.SyntaxErr,
-                    u'1 , ': xml.dom.SyntaxErr,
-                    u'1  ,  ': xml.dom.SyntaxErr,
-                    u'1//2': xml.dom.SyntaxErr,
+                    u',',
+                    u'1,,2',
+                    u'1,/**/,2',
+                    u'1  ,  /**/  ,  2',
+                    u'1,',
+                    u'1, ',
+                    u'1 ,',
+                    u'1 , ',
+                    u'1  ,  ',
+                    u'1//2',
                     # URL
-                    u'url(x))': xml.dom.SyntaxErr,
+                    u'url(x))',
                     # string
-                    u'"': xml.dom.SyntaxErr,
-                    u"'": xml.dom.SyntaxErr,
+                    u'"',
+                    u"'",
                     # function
-                    u'f(-)': xml.dom.SyntaxErr,
-                    u'f(x))': xml.dom.SyntaxErr
-                    }
-        self.do_raise_r(tests)
+                    u'f(-)',
+                    u'f(x))',
+                    #calc
+                    u'calc(',
+                    u'calc(1',
+                    u'calc(1 + 1',
+                    u'calc(1+1)',
+                    u'calc(1-1)',
+                    u'calc(1 +1)',
+                    u'calc(1+ 1)',
+                    u'calc(1 -1)',
+                    u'calc(1- 1)',
+                    u'calc(+)',
+                    u'calc(+ 1)',
+                    u'calc(-)',
+                    u'calc(- 1)',
+                    u'calc(*)',
+                    u'calc(*1)',
+                    u'calc(* 2)',
+                    u'calc(/)',
+                    u'calc(/1)',
+                    u'calc(/ 2)',
+                    u'calc(1+)',
+                    u'calc(1 +)',
+                    u'calc(1 + )',
+                    u'calc(2px -)',
+                    u'calc(3px*)',
+                    u'calc(3px *)',
+                    u'calc(3px * )',
+                    u'calc(4em/)',
+                    u'calc(4em /)',
+                    u'calc(4em / )',
+                    u'calc(1 + + 1)',
+                    u'calc(1 ++ 1)'
+                    ]
+        self.do_raise_r_list(tests, xml.dom.SyntaxErr)
 
     def test_list(self):
         "PropertyValue[index]"
@@ -300,13 +356,13 @@ class PropertyValueTestCase(basetest.BaseTestCase):
     def test_readonly(self):
         "PropertyValue._readonly"
         v = cssutils.css.PropertyValue(cssText='inherit')
-        self.assert_(False is v._readonly)
+        self.assertTrue(False is v._readonly)
 
         v = cssutils.css.PropertyValue(cssText='inherit', readonly=True)
-        self.assert_(True is v._readonly)
-        self.assert_(u'inherit', v.cssText)
+        self.assertTrue(True is v._readonly)
+        self.assertTrue(u'inherit', v.cssText)
         self.assertRaises(xml.dom.NoModificationAllowedErr, v._setCssText, u'x')
-        self.assert_(u'inherit', v.cssText)
+        self.assertTrue(u'inherit', v.cssText)
 
     def test_reprANDstr(self):
         "PropertyValue.__repr__(), .__str__()"
@@ -314,11 +370,11 @@ class PropertyValueTestCase(basetest.BaseTestCase):
 
         s = cssutils.css.PropertyValue(cssText=cssText)
 
-        self.assert_(cssText in str(s))
+        self.assertTrue(cssText in str(s))
 
         s2 = eval(repr(s))
-        self.assert_(isinstance(s2, s.__class__))
-        self.assert_(cssText == s2.cssText)
+        self.assertTrue(isinstance(s2, s.__class__))
+        self.assertTrue(cssText == s2.cssText)
 
 
 class ValueTestCase(basetest.BaseTestCase):
@@ -326,9 +382,9 @@ class ValueTestCase(basetest.BaseTestCase):
     def test_init(self):
         "Value.__init__()"
         v = cssutils.css.Value()
-        self.assert_(u'' == v.cssText)
-        self.assert_(u'' == v.value)
-        self.assert_(None is  v.type)
+        self.assertTrue(u'' == v.cssText)
+        self.assertTrue(u'' == v.value)
+        self.assertTrue(None is  v.type)
 
     def test_cssText(self):
         "Value.cssText"
@@ -367,8 +423,8 @@ class ColorValueTestCase(basetest.BaseTestCase):
         "ColorValue.__init__()"
         v = cssutils.css.ColorValue()
         self.assertEqual(v.COLOR_VALUE, v.type)
-        self.assert_(u'' == v.cssText)
-        self.assert_(u'' == v.value)
+        self.assertTrue(u'' == v.cssText)
+        self.assertTrue(u'' == v.value)
         self.assertEqual(u'transparent', v.name)
         self.assertEqual(None, v.colorType)
 
@@ -461,10 +517,10 @@ class ColorValueTestCase(basetest.BaseTestCase):
         for colors, rgba in tests.items():
             for color in colors:
                 c = cssutils.css.ColorValue(color);
-                self.assertEquals(c.red, rgba[0])
-                self.assertEquals(c.green, rgba[1])
-                self.assertEquals(c.blue, rgba[2])
-                self.assertEquals(c.alpha, rgba[3])
+                self.assertEqual(c.red, rgba[0])
+                self.assertEqual(c.green, rgba[1])
+                self.assertEqual(c.blue, rgba[2])
+                self.assertEqual(c.alpha, rgba[3])
 
 
 class URIValueTestCase(basetest.BaseTestCase):
@@ -472,19 +528,19 @@ class URIValueTestCase(basetest.BaseTestCase):
     def test_init(self):
         "URIValue.__init__()"
         v = cssutils.css.URIValue()
-        self.assert_(u'url()' == v.cssText)
-        self.assert_(u'' == v.value)
-        self.assert_(u'' == v.uri)
-        self.assert_(v.URI is  v.type)
+        self.assertTrue(u'url()' == v.cssText)
+        self.assertTrue(u'' == v.value)
+        self.assertTrue(u'' == v.uri)
+        self.assertTrue(v.URI is  v.type)
 
         v.uri = '1'
-        self.assert_(u'1' == v.value)
-        self.assert_(u'1' == v.uri)
+        self.assertTrue(u'1' == v.value)
+        self.assertTrue(u'1' == v.uri)
         self.assertEqual(u'url(1)', v.cssText)
 
         v.value = '2'
-        self.assert_(u'2' == v.value)
-        self.assert_(u'2' == v.uri)
+        self.assertTrue(u'2' == v.value)
+        self.assertTrue(u'2' == v.uri)
         self.assertEqual(u'url(2)', v.cssText)
 
     def test_absoluteUri(self):
@@ -540,10 +596,10 @@ class DimensionValueTestCase(basetest.BaseTestCase):
     def test_init(self):
         "DimensionValue.__init__()"
         v = cssutils.css.DimensionValue()
-        self.assert_(u'' == v.cssText)
-        self.assert_(u'' == v.value)
-        self.assert_(None is  v.type)
-        self.assert_(None is  v.dimension)
+        self.assertTrue(u'' == v.cssText)
+        self.assertTrue(u'' == v.value)
+        self.assertTrue(None is  v.type)
+        self.assertTrue(None is  v.dimension)
 
     def test_cssText(self):
         "DimensionValue.cssText"
@@ -635,16 +691,22 @@ class CSSVariableTestCase(basetest.BaseTestCase):
         v = cssutils.css.CSSVariable()
         self.assertEqual(u'', v.cssText)
         self.assertEqual(u'VARIABLE', v.type)
-        self.assert_(None is v.name)
-        self.assert_(None is v.value)
+        self.assertTrue(None is v.name)
+        self.assertTrue(None is v.value)
 
     def test_cssText(self):
         "CSSVariable.cssText"
         tests = {
-                 u'var(x)': (u'var(x)', 'x'),
-                 u'VAR(  X  )': (u'var(X)', 'X')
+                 u'var(x)': (u'var(x)', u'x', None),
+                 u'VAR(  X  )': (u'var(X)', u'X', None),
+                 u'var(c1,rgb(14,14,14))': (u'var(c1, rgb(14, 14, 14))', u'c1', u'rgb(14, 14, 14)'),
+                 u'var( L, 1px )': (u'var(L, 1px)', u'L', u'1px'),
+                 u'var(L,1)': (u'var(L, 1)', u'L', u'1'),
+                 u'var(T, calc( 2 * 1px ))': (u'var(T, calc(2 * 1px))', u'T', 'calc(2 * 1px)'),
+                 u'var(U, url( example.png ) )': (u'var(U, url(example.png))', u'U', u'url(example.png)'),
+                 u'var(C, #f00 )': (u'var(C, #f00)', u'C', '#fff')
                  }
-        for (var, (cssText, name)) in tests.items():
+        for (var, (cssText, name, fallback)) in tests.items():
             v = cssutils.css.CSSVariable(var)
             self.assertEqual(cssText, v.cssText)
             self.assertEqual('VARIABLE', v.type)
@@ -687,13 +749,13 @@ class CSSVariableTestCase(basetest.BaseTestCase):
 #    def test_init(self):
 #        "CSSPrimitiveValue.__init__()"
 #        v = cssutils.css.CSSPrimitiveValue(u'1')
-#        self.assert_(u'1' == v.cssText)
+#        self.assertTrue(u'1' == v.cssText)
 #
-#        self.assert_(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
-#        self.assert_("CSS_PRIMITIVE_VALUE" == v.cssValueTypeString)
+#        self.assertTrue(v.CSS_PRIMITIVE_VALUE == v.cssValueType)
+#        self.assertTrue("CSS_PRIMITIVE_VALUE" == v.cssValueTypeString)
 #
-#        self.assert_(v.CSS_NUMBER == v.primitiveType)
-#        self.assert_("CSS_NUMBER" == v.primitiveTypeString)
+#        self.assertTrue(v.CSS_NUMBER == v.primitiveType)
+#        self.assertTrue("CSS_NUMBER" == v.primitiveTypeString)
 #
 #        # DUMMY to be able to test empty constructor call
 #        #self.assertRaises(xml.dom.SyntaxErr, v.__init__, None)
@@ -706,8 +768,8 @@ class CSSVariableTestCase(basetest.BaseTestCase):
 #    def test_CSS_UNKNOWN(self):
 #        "CSSPrimitiveValue.CSS_UNKNOWN"
 #        v = cssutils.css.CSSPrimitiveValue(u'expression(false)')
-#        self.assert_(v.CSS_UNKNOWN == v.primitiveType)
-#        self.assert_('CSS_UNKNOWN' == v.primitiveTypeString)
+#        self.assertTrue(v.CSS_UNKNOWN == v.primitiveType)
+#        self.assertTrue('CSS_UNKNOWN' == v.primitiveTypeString)
 #
 #    def test_CSS_NUMBER_AND_OTHER_DIMENSIONS(self):
 #        "CSSPrimitiveValue.CSS_NUMBER .. CSS_DIMENSION"
@@ -924,7 +986,7 @@ class CSSVariableTestCase(basetest.BaseTestCase):
 #    def test_getString(self):
 #        "CSSPrimitiveValue.getStringValue()"
 #        v = cssutils.css.CSSPrimitiveValue(u'1px')
-#        self.assert_(v.primitiveType == v.CSS_PX)
+#        self.assertTrue(v.primitiveType == v.CSS_PX)
 #        self.assertRaises(xml.dom.InvalidAccessErr,
 #                          v.getStringValue)
 #
@@ -956,9 +1018,9 @@ class CSSVariableTestCase(basetest.BaseTestCase):
 #        "CSSPrimitiveValue.setStringValue()"
 #        # CSS_STRING
 #        v = cssutils.css.CSSPrimitiveValue(u'"a"')
-#        self.assert_(v.CSS_STRING == v.primitiveType)
+#        self.assertTrue(v.CSS_STRING == v.primitiveType)
 #        v.setStringValue(v.CSS_STRING, 'b')
-#        self.assert_(('b', 'STRING') == v._value)
+#        self.assertTrue(('b', 'STRING') == v._value)
 #        self.assertEqual('b', v.getStringValue())
 #        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
 #            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_STRING' to 'CSS_URI'",
@@ -973,9 +1035,9 @@ class CSSVariableTestCase(basetest.BaseTestCase):
 #        # CSS_IDENT
 #        v = cssutils.css.CSSPrimitiveValue('new')
 #        v.setStringValue(v.CSS_IDENT, 'ident')
-#        self.assert_(v.CSS_IDENT == v.primitiveType)
-#        self.assert_(('ident', 'IDENT') == v._value)
-#        self.assert_('ident' == v.getStringValue())
+#        self.assertTrue(v.CSS_IDENT == v.primitiveType)
+#        self.assertTrue(('ident', 'IDENT') == v._value)
+#        self.assertTrue('ident' == v.getStringValue())
 #        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
 #            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_IDENT' to 'CSS_URI'",
 #            v.setStringValue, *(v.CSS_URI, 'x'))
@@ -1017,7 +1079,7 @@ class CSSVariableTestCase(basetest.BaseTestCase):
 #        self.assertEqual(u'a)', v.getStringValue())
 #
 #        v.setStringValue(v.CSS_URI, 'a')
-#        self.assert_(v.CSS_URI == v.primitiveType)
+#        self.assertTrue(v.CSS_URI == v.primitiveType)
 #        self.assertEqual((u'a', 'URI'), v._value)
 #        self.assertEqual(u'a', v.getStringValue())
 #
@@ -1034,8 +1096,8 @@ class CSSVariableTestCase(basetest.BaseTestCase):
 #        # CSS_ATTR
 #        v = cssutils.css.CSSPrimitiveValue('attr(old)')
 #        v.setStringValue(v.CSS_ATTR, 'a')
-#        self.assert_(v.CSS_ATTR == v.primitiveType)
-#        self.assert_('a' == v.getStringValue())
+#        self.assertTrue(v.CSS_ATTR == v.primitiveType)
+#        self.assertTrue('a' == v.getStringValue())
 #        self.assertRaisesMsg(xml.dom.InvalidAccessErr,
 #            u"CSSPrimitiveValue: Cannot coerce primitiveType 'CSS_ATTR' to 'CSS_IDENT'",
 #            v.setStringValue, *(v.CSS_IDENT, 'x'))
