@@ -2,23 +2,24 @@
 """
 __all__ = []
 __docformat__ = 'restructuredtext'
-__version__ = '$Id: util.py 1781 2009-07-19 12:30:49Z cthedot $'
+__version__ = '$Id$'
 
 from helper import normalize
-from itertools import ifilter
-import css
+from itertools import ifilter, chain
+import cssutils
 import codec
 import codecs
 import errorhandler
 import tokenize2
 import types
 import xml.dom
+import re
 
 try:
     from _fetchgae import _defaultFetcher
 except ImportError, e:
     from _fetch import _defaultFetcher
-    
+
 log = errorhandler.ErrorHandler()
 
 class _BaseClass(object):
@@ -66,6 +67,9 @@ class _NewBase(_BaseClass):
         newseq._readonly = True
         self._seq = newseq
 
+    def _clearSeq(self):
+        self._seq.clear()
+
     def _tempSeq(self, readonly=False):
         "Get a writeable Seq() which is used to set ``seq`` later"
         return Seq(readonly=readonly)
@@ -74,6 +78,54 @@ class _NewBase(_BaseClass):
                    doc="Internal readonly attribute, **DO NOT USE**!")
 
 
+class _NewListBase(_NewBase):
+    """
+    (EXPERIMENTAL)
+    A base class used for list classes like stylesheets.MediaList
+
+    adds list like behaviour running on inhering class' property ``seq``
+
+    - item in x => bool
+    - len(x) => integer
+    - get, set and del x[i]
+    - for item in x
+    - append(item)
+
+    some methods must be overwritten in inheriting class
+    """
+    def __init__(self):
+        self._seq = Seq()
+
+    def __contains__(self, item):
+        for it in self._seq:
+            if item == it.value:
+                return True
+        return False
+
+    def __delitem__(self, index):
+        del self._seq[index]
+
+    def __getitem__(self, index):
+        return self._seq[index].value
+
+    def __iter__(self):
+        def gen():
+            for x in self._seq:
+                yield x.value
+        return gen()
+
+    def __len__(self):
+        return len(self._seq)
+
+    def __setitem__(self, index, item):
+        "must be overwritten"
+        raise NotImplementedError
+
+    def append(self, item):
+        "must be overwritten"
+        raise NotImplementedError
+
+
 class Base(_BaseClass):
     """
     **Superceded by _NewBase**
@@ -93,7 +145,7 @@ class Base(_BaseClass):
     # format: shorthand: [(propname, mandatorycheck?)*]
     _SHORTHANDPROPERTIES = {
             u'background': [],
-            u'background-position': [],
+            #u'background-position': [], # list of 2 values!
             u'border': [],
             u'border-left': [],
             u'border-right': [],
@@ -148,21 +200,19 @@ class Base(_BaseClass):
             # needs to be tokenized
             return self.__tokenizer2.tokenize(
                  textortokens)
-        elif types.GeneratorType == type(textortokens):
-            # already tokenized
-            return textortokens
         elif isinstance(textortokens, tuple):
             # a single token (like a comment)
             return [textortokens]
         else:
-            # already tokenized but return generator
-            return (x for x in textortokens)
+            # already tokenized but return an iterator
+            return iter(textortokens)
 
     def _nexttoken(self, tokenizer, default=None):
         "returns next token in generator tokenizer or the default value"
         try:
             return tokenizer.next()
-        except (StopIteration, AttributeError):
+        # TypeError for py3
+        except (StopIteration, AttributeError, TypeError):
             return default
 
     def _type(self, token):
@@ -277,12 +327,21 @@ class Base(_BaseClass):
         resulttokens = []
         if starttoken:
             resulttokens.append(starttoken)
+            val = starttoken[1]
+            if u'[' == val:
+                bracket += 1
+            elif u'{' == val:
+                brace += 1
+            elif u'(' == val:
+                parant += 1
+
         if tokenizer:
             for token in tokenizer:
                 typ, val, line, col = token
                 if 'EOF' == typ:
                     resulttokens.append(token)
                     break
+
                 if u'{' == val:
                     brace += 1
                 elif u'}' == val:
@@ -330,7 +389,7 @@ class Base(_BaseClass):
             "default impl for unexpected @rule"
             if expected != 'EOF':
                 # TODO: parentStyleSheet=self
-                rule = css.CSSUnknownRule()
+                rule = cssutils.css.CSSUnknownRule()
                 rule.cssText = self._tokensupto2(tokenizer, token)
                 if rule.wellformed:
                     seq.append(rule)
@@ -342,7 +401,7 @@ class Base(_BaseClass):
 
         def COMMENT(expected, seq, token, tokenizer=None):
             "default implementation for COMMENT token adds CSSCommentRule"
-            seq.append(css.CSSComment([token]))
+            seq.append(cssutils.css.CSSComment([token]))
             return expected
 
         def S(expected, seq, token, tokenizer=None):
@@ -394,7 +453,7 @@ class Base(_BaseClass):
                 yield initialtoken
                 for item in tokenizer:
                     yield item
-            fulltokenizer = (t for t in tokens())
+            fulltokenizer = chain([initialtoken], tokenizer)
         else:
             fulltokenizer = tokenizer
 
@@ -433,10 +492,10 @@ class Base2(Base, _NewBase):
             "default impl for unexpected @rule"
             if expected != 'EOF':
                 # TODO: parentStyleSheet=self
-                rule = css.CSSUnknownRule()
+                rule = cssutils.css.CSSUnknownRule()
                 rule.cssText = self._tokensupto2(tokenizer, token)
                 if rule.wellformed:
-                    seq.append(rule, css.CSSRule.UNKNOWN_RULE,
+                    seq.append(rule, cssutils.css.CSSRule.UNKNOWN_RULE,
                                line=token[2], col=token[3])
                 return expected
             else:
@@ -449,7 +508,7 @@ class Base2(Base, _NewBase):
             if expected == 'EOF':
                 new['wellformed'] = False
                 self._log.error(u'Expected EOF but found comment.', token=token)
-            seq.append(css.CSSComment([token]), 'COMMENT')
+            seq.append(cssutils.css.CSSComment([token]), 'COMMENT')
             return expected
 
         def S(expected, seq, token, tokenizer=None):
@@ -488,6 +547,27 @@ class Seq(object):
         self._seq = []
         self._readonly = readonly
 
+    def __repr__(self):
+        "returns a repr same as a list of tuples of (value, type)"
+        return u'cssutils.%s.%s([\n    %s], readonly=%r)' % (self.__module__,
+                                          self.__class__.__name__,
+            u',\n    '.join([u'%r' % item for item in self._seq]
+            ), self._readonly)
+
+    def __str__(self):
+        vals = []
+        for v in self:
+            if isinstance(v.value, basestring):
+                vals.append(v.value)
+            elif isinstance(v, tuple):
+                vals.append(v.value[1])
+            else:
+                vals.append(repr(v))
+
+        return "<cssutils.%s.%s object length=%r items=%r readonly=%r at 0x%x>" % (
+                self.__module__, self.__class__.__name__, len(self),
+                vals, self._readonly, id(self))
+
     def __delitem__(self, i):
         del self._seq[i]
 
@@ -503,12 +583,15 @@ class Seq(object):
     def __len__(self):
         return len(self._seq)
 
-    def append(self, val, typ, line=None, col=None):
-        "if not readonly add new Item()"
+    def append(self, val, typ=None, line=None, col=None):
+        "If not readonly add new Item()"
         if self._readonly:
             raise AttributeError('Seq is readonly.')
         else:
-            self._seq.append(Item(val, typ, line, col))
+            if isinstance(val, Item):
+                self._seq.append(val)
+            else:
+                self._seq.append(Item(val, typ, line, col))
 
     def appendItem(self, item):
         "if not readonly add item which must be an Item"
@@ -517,7 +600,14 @@ class Seq(object):
         else:
             self._seq.append(item)
 
-    def replace(self, index= - 1, val=None, typ=None, line=None, col=None):
+    def clear(self):
+        del self._seq[:]
+
+    def insert(self, index, val, typ, line=None, col=None):
+        "Insert new Item() at index # even if readony!? TODO!"
+        self._seq.insert(index, Item(val, typ, line, col))
+    
+    def replace(self, index=-1, val=None, typ=None, line=None, col=None):
         """
         if not readonly replace Item at index with new Item or
         simply replace value or type
@@ -544,26 +634,6 @@ class Seq(object):
             self._seq[index] = Item(old.value + val, old.type,
                                     old.line, old.col)
 
-    def __repr__(self):
-        "returns a repr same as a list of tuples of (value, type)"
-        return u'cssutils.%s.%s([\n    %s], readonly=%r)' % (self.__module__,
-                                          self.__class__.__name__,
-            u',\n    '.join([u'%r' % item for item in self._seq]
-            ), self._readonly)
-
-    def __str__(self):
-        vals = []
-        for v in self:
-            if isinstance(v.value, basestring):
-                vals.append(v.value)
-            elif type(v) == tuple:
-                vals.append(v.value[1])
-            else:
-                vals.append(str(v))
-
-        return "<cssutils.%s.%s object length=%r values=%r readonly=%r at 0x%x>" % (
-                self.__module__, self.__class__.__name__, len(self),
-                u''.join(vals), self._readonly, id(self))
 
 class Item(object):
     """
@@ -599,7 +669,7 @@ class Item(object):
 class ListSeq(object):
     """
     (EXPERIMENTAL)
-    A base class used for list classes like css.SelectorList or
+    A base class used for list classes like cssutils.css.SelectorList or
     stylesheets.MediaList
 
     adds list like behaviour running on inhering class' property ``seq``
@@ -683,14 +753,14 @@ class _Namespaces(object):
                 self.parentStyleSheet.deleteRule(i)
                 return
 
-        self._log.error('Prefix %r not found.' % prefix,
+        self._log.error('Prefix %s not found.' % prefix,
                         error=xml.dom.NamespaceErr)
 
     def __getitem__(self, prefix):
         try:
             return self.namespaces[prefix]
         except KeyError, e:
-            self._log.error('Prefix %r not found.' % prefix,
+            self._log.error('Prefix %s not found.' % prefix,
                             error=xml.dom.NamespaceErr)
 
     def __iter__(self):
@@ -705,7 +775,7 @@ class _Namespaces(object):
             prefix = u'' # None or ''
         rule = self.__findrule(prefix)
         if not rule:
-            self.parentStyleSheet.insertRule(css.CSSNamespaceRule(
+            self.parentStyleSheet.insertRule(cssutils.css.CSSNamespaceRule(
                                                     prefix=prefix,
                                                     namespaceURI=namespaceURI),
                                   inOrder=True)
@@ -754,7 +824,7 @@ class _Namespaces(object):
         for prefix, uri in self.namespaces.items():
             if uri == namespaceURI:
                 return prefix
-        raise IndexError(u'NamespaceURI %r not found.' % namespaceURI)
+        raise IndexError(u'NamespaceURI %s not found.' % namespaceURI)
 
     def __str__(self):
         return u"<cssutils.util.%s object parentStyleSheet=%r at 0x%x>" % (
@@ -797,19 +867,20 @@ def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
     - decodedCssText
 
     ``fetcher``
-        see cssutils.registerFetchUrl for details
+        see cssutils.CSSParser.setFetcher for details
     ``overrideEncoding``
         If given this encoding is used and all other encoding information is
         ignored (HTTP, BOM etc)
     ``parentEncoding``
-        Encoding of parent stylesheet (while e.g. reading @import references sheets)
-        or document if available.
+        Encoding of parent stylesheet (while e.g. reading @import references
+        sheets) or document if available.
 
     Priority or encoding information
     --------------------------------
     **cssutils only**: 0. overrideEncoding
 
-    1. An HTTP "charset" parameter in a "Content-Type" field (or similar parameters in other protocols)
+    1. An HTTP "charset" parameter in a "Content-Type" field (or similar
+       parameters in other protocols)
     2. BOM and/or @charset (see below)
     3. <link charset=""> or other metadata from the linking mechanism (if any)
     4. charset of referring style sheet or document (if any)
@@ -820,6 +891,7 @@ def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
 
     if not fetcher:
         fetcher = _defaultFetcher
+
     r = fetcher(url)
     if r and len(r) == 2 and r[1] is not None:
         httpEncoding, content = r
@@ -830,21 +902,22 @@ def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
         elif httpEncoding:
             enctype = 1 # 1. HTTP
             encoding = httpEncoding
-        else:  
-            if isinstance(content, unicode): 
-                # no need to check content as unicode so no BOM
-                explicit = False
+        else:
+            # BOM or @charset
+            if isinstance(content, unicode):
+                contentEncoding, explicit = codec.detectencoding_unicode(content)
             else:
-                # check content
                 contentEncoding, explicit = codec.detectencoding_str(content)
-
+            
             if explicit:
                 enctype = 2 # 2. BOM/@charset: explicitly
                 encoding = contentEncoding
+                
             elif parentEncoding:
                 enctype = 4 # 4. parent stylesheet or document
                 # may also be None in which case 5. is used in next step anyway
                 encoding = parentEncoding
+                
             else:
                 enctype = 5 # 5. assume UTF-8
                 encoding = 'utf-8'
@@ -854,7 +927,12 @@ def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
         else:
             try:
                 # encoding may still be wrong if encoding *is lying*!
-                decodedCssText = codecs.lookup("css")[1](content, encoding=encoding)[0]
+                try:
+                    decodedCssText = codecs.lookup("css")[1](content, encoding=encoding)[0]
+                except AttributeError, ae:
+                    # at least in GAE
+                    decodedCssText = content.decode(encoding if encoding else 'utf-8')
+                    
             except UnicodeDecodeError, e:
                 log.warn(e, neverraise=True)
                 decodedCssText = None
@@ -863,3 +941,119 @@ def _readUrl(url, fetcher=None, overrideEncoding=None, parentEncoding=None):
     else:
         return None, None, None
 
+
+class LazyRegex(object):
+    """A class to represent a lazily compiled regular expression.
+
+    The interface is kept similar to a `re.compile`ed object from the standard
+    regex library.
+
+    :ivar pattern: The original regular expression.
+    :ivar flags: Flags of the regular expression.
+    :ivar matcher: The compiled regular expression, or None if it is not yet
+                   compiled.
+    :ivar groups: The number of capturing groups in the pattern, or None.
+    :ivar groupindex: A dictionary mapping any symbolic group names defined by
+                      `(?P<id>)` to group numbers. The dictionary is empty if
+                      no symbolic groups were used in the pattern. None if the
+                      pattern is not yet compiled.
+    """
+
+    __slots__ = ('pattern', 'flags', 'matcher', 'groups', 'groupindex')
+
+    def __init__(self, pattern, flags=0):
+        self.pattern = pattern
+        self.matcher = None
+        self.flags = flags
+        self.groups = None
+        self.groupindex = None
+
+    def ensure(self):
+        """Make sure that the expression is compiled.
+
+        If self.matcher is already a compiled expression, do nothing.
+        """
+        if self.matcher is not None:
+            return
+        self.matcher = re.compile(self.pattern, self.flags)
+        self.flags = self.matcher.flags
+        self.groups = self.matcher.groups
+        self.groupindex = self.matcher.groupindex
+
+    def __call__(self, string, pos=None, endpos=None):
+        """Shortcut for self.match(string)."""
+        return self.match(string, pos, endpos)
+
+    def match(self, string, pos=None, endpos=None):
+        """Attempt to do a match of the regular expression.
+
+        This is similar to the `.match` method of `re` objects.
+        """
+        self.ensure()
+        if pos is None:
+            pos = 0
+        if endpos is None:
+            endpos = len(string)
+        return self.matcher.match(string, pos, endpos)
+
+    def search(self, string, pos=None, endpos=None):
+        """Search the string for the pattern.
+
+        This is similar to the `.search` method of `re` objects.
+        """
+        self.ensure()
+        if pos is None:
+            pos = 0
+        if endpos is None:
+            endpos = len(string)
+        return self.matcher.search(string, pos, endpos)
+
+    def split(self, string, maxsplit=0):
+        """Split the string at the pattern.
+
+        This is similar to the `.split` method of `re` objects.
+        """
+        self.ensure()
+        return self.matcher.split(string, maxsplit)
+
+    def findall(self, string, pos=None, endpos=None):
+        """Find all instances of the pattern in the given string.
+
+        This is similar to the `.findall` method of `re` objects.
+        """
+        self.ensure()
+        if pos is None:
+            pos = 0
+        if endpos is None:
+            endpos = len(string)
+        return self.matcher.findall(string, pos, endpos)
+
+    def finditer(self, string, pos=None, endpos=None):
+        """Find all instances of the pattern in the given string.
+
+        Returns an iterator.
+
+        This is similar to the `.finditer` method of `re` objects.
+        """
+        self.ensure()
+        if pos is None:
+            pos = 0
+        if endpos is None:
+            endpos = len(string)
+        return self.matcher.finditer(string, pos, endpos)
+
+    def sub(self, repl, string, count=0):
+        """Replace all occurences of the pattern with the given replacement.
+
+        This is similar to the `.sub` method of `re` objects.
+        """
+        self.ensure()
+        return self.matcher.sub(repl, string, count)
+
+    def subn(self, repl, string, count=0):
+        """Replace all occurences of the pattern with the given replacement.
+
+        This is similar to the `.subn` method of `re` objects.
+        """
+        self.ensure()
+        return self.matcher.subn(repl, string, count)
